#ifndef _OKAN_ORTO_H_
#define _OKAN_ORTO_H_

#include "OkanMisc.h"
#include <cmath>
#include <vector>

#define OKAN_PI 3.14159265

/**
 * Image Parameter class. It contains all image parameters.
 *
 * @author Yunus OLGUN
 */
class OkanImageParam
{
	public:
/**
 * Public member which keeps focus distance of camera in meters.
 */
		double f;
/**
 * Public member which keeps pixel size of camera in meters.
 */
		double pixelSize;
/**
 * Public member which keeps principal point of camera in x-axis.
 */
		int principalPointX;
/**
 * Public member which keeps principal point of camera in y-axis.
 */
		int principalPointY;
/**
 * Public member which keeps omega of camera in radian.
 */
		double omega;
/**
 * Public member which keeps phi of camera in radian.
 */
		double phi;
/**
 * Public member which keeps kappa of camera in radian.
 */
		double kappa;
/**
 * Public member which keeps X of camera in meters.
 */
		double camX;
/**
 * Public member which keeps Y of camera in meters.
 */
		double camY;
/**
 * Public member which keeps Z of camera in meters.
 */
		double camZ;

/**
 * Default constuctor. It does not have any effect.
 */
		OkanImageParam(){}
/**
 * Constuctor. It sets every element of class.
 * @param f give it in milimeters
 * @param pixelSize give it in microns
 * @param principalPointX should be integer
 * @param principalPointY should be integer
 * @param omega give it in degrees
 * @param phi give it in degrees
 * @param kappa give it in degrees
 * @param camX give it in meters
 * @param camY give it in meters
 * @param camZ give it in meters
 */
		OkanImageParam( double f , double pixelSize , int principalPointX , int principalPointY , double omega , double phi , double kappa , double camX , double camY , double camZ )
		{
			this->f = f / 1000;
			this->pixelSize = pixelSize / 1000000;
			this->principalPointX = principalPointX;
			this->principalPointY = principalPointY;
			this->omega = omega * OKAN_PI / 180;
			this->phi = phi * OKAN_PI / 180;
			this->kappa = kappa * OKAN_PI / 180;
			this->camX = camX;
			this->camY = camY;
			this->camZ = camZ;
		}
/**
 * Default copy constuctor.
 * @param rhs the class which will be copied into the new one.
 */
		OkanImageParam( const OkanImageParam & rhs )
		{
			*this = rhs;
		}

/**
 * Default assignment operator.
 * @param rhs right hand side of assignment.
 */
		OkanImageParam & operator=( const OkanImageParam & rhs )
		{
			this->f = rhs.f;
			this->pixelSize = rhs.pixelSize;
			this->principalPointX = rhs.principalPointX;
			this->principalPointY = rhs.principalPointY;
			this->omega = rhs.omega;
			this->phi = rhs.phi;
			this->kappa = rhs.kappa;
			this->camX = rhs.camX;
			this->camY = rhs.camY;
			this->camZ = rhs.camZ;
			return *this;
		}
};

/**
 * Ortophoto generator class.
 *
 * @author Yunus OLGUN
 */
class OkanOrto
{
	public:
/**
 * Private member which keeps the base image.
 */
		vector<IplImage*> image;
/**
 * Private member which keeps inputMask of the base image.
 */
		vector<uchar**> inputMask;
/**
 * Private member which keeps image parameters
 */
		vector<OkanImageParam> imageParam;
/**
 * Private member which keeps X values of image pixels.
 */
		vector<double**> ortoX;
/**
 * Private member which keeps Y values of image pixels.
 */
		vector<double**> ortoY;
/**
 * Private member which keeps minimum values of image pixels. min[0] is in x-axis. min[1] is in y-axis.
 */
		double min[2];
/**
 * Private member which keeps maximum values of image pixels. max[0] is in x-axis. max[1] is in y-axis.
 */
		double max[2];

/**
 * Private method which is used for generating an image from ortophoto data
 */
		void coreGiveOrtoImage( IplImage* & newImage, uchar** & outputMask , double line[6] , int newWidth , int newHeight , bool stereo );

	public:
/**
 * Default constructor. It assigns NULL to pointers.
 */
		OkanOrto();
/**
 * Default destructor. It deletes ortoX and ortoY.
 */
		~OkanOrto();

/**
 * @TODO: addexplanation
 */
		void clear();

/**
 * It is used for setting image. It takes copies of im and inputMask.
 * @param im image which will be copied. It only copies pointer of image
 * @param new_inputMask input mask of image. It only copies pointer of mask. If it is NULL that means every pixel will be used at generating ortophoto
 * @param new_imageParam parameters of image
 */
		void setImage( IplImage* im , uchar **new_inputMask, const OkanImageParam & new_imageParam , double new_avgHeight , double **dem = NULL );

/**
 * It creates a new image by using ortoX, ortoY and image fields. newImage's size will be same as the base image.
 * @param newImage It is used for output. Output image will be saved at this. Size of output will be the same as the base image. It should not be created before calling this function. It will be created in function.
 * @param outputMask mask of output. Every invalid pixel will be 0. Others will be more than 0. Actually every pixel will show how many pixels of base image matched to that point
 * @param line it is used for creating world file. For creating a world file for output you can use this one. Simply prints every value as a line
 */

		void giveOrtoImage( IplImage* & newImage, uchar** & outputMask , double line[6] , bool stereo = false );
/**
 * It creates a new image by using ortoX, ortoY and image fields. newImage's size will be calculated according to pixelSize at x and y axises.
 * @param newImage It is used for output. Output image will be saved at this. Size of output will be the same as the base image. It should not be created before calling this function. It will be created in function.
 * @param outputMask mask of output. Every invalid pixel will be 0. Others will be more than 0. Actually every pixel will show how many pixels of base image matched to that point
 * @param line it is used for creating world file. For creating a world file for output you can use this one. Simply prints every value as a line
 * @param pixelSizeX new image's pixelSize in x-axis.
 * @param pixelSizeY new image's pixelSize in y-axis.
 */
		void giveOrtoImageByPixelSize( IplImage* & newImage , uchar** & outputMask , double line[6] , double pixelSizeX , double pixelSizeY , bool stereo = false );

/**
 * It creates a new image by using ortoX, ortoY and image fields. newImage's size will be calculated according to pixelSize at x and y axises.
 * @param newImage It is used for output. Output image will be saved at this. Size of output will be the same as the base image. It should not be created before calling this function. It will be created in function.
 * @param outputMask mask of output. Every invalid pixel will be 0. Others will be more than 0. Actually every pixel will show how many pixels of base image matched to that point
 * @param line it is used for creating world file. For creating a world file for output you can use this one. Simply prints every value as a line
 * @param resX width of new image
 * @param resY height of new image
 */
		void giveOrtoImageByResolution( IplImage* & newImage , uchar** & outputMask , double line[6] , int resX , int resY , bool stereo = false );
};

#endif // OkanOrto.h
